@@ -1,4 +1,3 @@
-#!/usr/bin/env python2.7
 # Copyright 2017 gRPC authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -113,6 +112,10 @@ LANG_RELEASE_MATRIX = {
             ('v1.35.0', ReleaseInfo()),
             ('v1.36.3', ReleaseInfo()),
             ('v1.37.0', ReleaseInfo()),
+            ('v1.38.0', ReleaseInfo()),
+            ('v1.39.0', ReleaseInfo()),
+            ('v1.41.1', ReleaseInfo()),
+            ('v1.42.0', ReleaseInfo()),
         ]),
     'go':
         OrderedDict([
@@ -187,6 +190,11 @@ LANG_RELEASE_MATRIX = {
             ('v1.36.0', ReleaseInfo(runtimes=['go1.11'])),
             ('v1.37.0', ReleaseInfo(runtimes=['go1.11'])),
             # NOTE: starting from release v1.38.0, use runtimes=['go1.16']
+            ('v1.38.1', ReleaseInfo(runtimes=['go1.16'])),
+            ('v1.39.1', ReleaseInfo(runtimes=['go1.16'])),
+            ('v1.40.0', ReleaseInfo(runtimes=['go1.16'])),
+            ('v1.41.0', ReleaseInfo(runtimes=['go1.16'])),
+            ('v1.42.0', ReleaseInfo(runtimes=['go1.16'])),
         ]),
     'java':
         OrderedDict([
@@ -257,71 +265,139 @@ LANG_RELEASE_MATRIX = {
             ('v1.34.1', ReleaseInfo()),
             ('v1.35.1', ReleaseInfo()),
             ('v1.36.1', ReleaseInfo()),
-            ('v1.37.0', ReleaseInfo()),
+            ('v1.37.1', ReleaseInfo()),
+            ('v1.38.1', ReleaseInfo()),
+            ('v1.39.0', ReleaseInfo()),
+            ('v1.40.1', ReleaseInfo()),
+            ('v1.40.2', ReleaseInfo()),
+            ('v1.41.0', ReleaseInfo()),
+            ('v1.41.1', ReleaseInfo()),
+            ('v1.42.1', ReleaseInfo()),
         ]),
     'python':
-        OrderedDict([
-            ('v1.0.x',
-             ReleaseInfo(runtimes=['python'], testcases_file='python__v1.0.x')),
-            ('v1.1.4',
-             ReleaseInfo(runtimes=['python'], testcases_file='python__v1.0.x')),
-            ('v1.2.5',
-             ReleaseInfo(runtimes=['python'], testcases_file='python__v1.0.x')),
-            ('v1.3.9',
-             ReleaseInfo(runtimes=['python'], testcases_file='python__v1.0.x')),
-            ('v1.4.2',
-             ReleaseInfo(runtimes=['python'], testcases_file='python__v1.0.x')),
-            ('v1.6.6',
-             ReleaseInfo(runtimes=['python'], testcases_file='python__v1.0.x')),
-            ('v1.7.2',
-             ReleaseInfo(runtimes=['python'], testcases_file='python__v1.0.x')),
-            ('v1.8.1',
-             ReleaseInfo(runtimes=['python'], testcases_file='python__v1.0.x')),
-            ('v1.9.1',
-             ReleaseInfo(runtimes=['python'], testcases_file='python__v1.0.x')),
-            ('v1.10.1',
-             ReleaseInfo(runtimes=['python'], testcases_file='python__v1.0.x')),
-            ('v1.11.1',
-             ReleaseInfo(runtimes=['python'],
-                         testcases_file='python__v1.11.1')),
-            ('v1.12.0',
-             ReleaseInfo(runtimes=['python'],
-                         testcases_file='python__v1.11.1')),
-            ('v1.13.0',
-             ReleaseInfo(runtimes=['python'],
-                         testcases_file='python__v1.11.1')),
-            ('v1.14.1',
-             ReleaseInfo(runtimes=['python'],
-                         testcases_file='python__v1.11.1')),
-            ('v1.15.0',
-             ReleaseInfo(runtimes=['python'],
-                         testcases_file='python__v1.11.1')),
-            ('v1.16.0',
-             ReleaseInfo(runtimes=['python'],
-                         testcases_file='python__v1.11.1')),
-            ('v1.17.1',
-             ReleaseInfo(runtimes=['python'],
-                         testcases_file='python__v1.11.1')),
-            ('v1.18.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.19.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.20.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.21.4', ReleaseInfo(runtimes=['python'])),
-            ('v1.22.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.22.1', ReleaseInfo(runtimes=['python'])),
-            ('v1.23.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.24.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.25.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.26.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.27.3', ReleaseInfo(runtimes=['python'])),
-            ('v1.30.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.31.1', ReleaseInfo(runtimes=['python'])),
-            ('v1.32.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.33.2', ReleaseInfo(runtimes=['python'])),
-            ('v1.34.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.35.0', ReleaseInfo(runtimes=['python'])),
-            ('v1.36.3', ReleaseInfo(runtimes=['python'])),
-            ('v1.37.0', ReleaseInfo(runtimes=['python'])),
-        ]),
+        OrderedDict(
+            [
+                ('v1.0.x',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.0.x')),
+                ('v1.1.4',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.0.x')),
+                ('v1.2.5',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.0.x')),
+                ('v1.3.9',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.0.x')),
+                ('v1.4.2',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.0.x')),
+                ('v1.6.6',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.0.x')),
+                ('v1.7.2',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.0.x')),
+                ('v1.8.1',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.0.x')),
+                ('v1.9.1',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.0.x')),
+                ('v1.10.1',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.0.x')),
+                ('v1.11.1',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.11.1')),
+                ('v1.12.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.11.1')),
+                ('v1.13.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.11.1')),
+                ('v1.14.1',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.11.1')),
+                ('v1.15.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.11.1')),
+                ('v1.16.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.11.1')),
+                ('v1.17.1',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.11.1')),
+                ('v1.18.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.19.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.20.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.21.4',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.22.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.22.1',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.23.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.24.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.25.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.26.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.27.3',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.30.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.31.1',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.32.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.33.2',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.34.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.35.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.36.3',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.37.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.38.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.39.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.18.0')),
+                ('v1.41.1',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.41.1')),
+                ('v1.42.0',
+                 ReleaseInfo(runtimes=['python'],
+                             testcases_file='python__v1.41.1')),
+            ]),
     'node':
         OrderedDict([
             ('v1.0.1', ReleaseInfo(testcases_file='node__v1.0.1')),
@@ -387,6 +463,10 @@ LANG_RELEASE_MATRIX = {
             ('v1.35.0', ReleaseInfo()),
             ('v1.36.3', ReleaseInfo()),
             ('v1.37.0', ReleaseInfo()),
+            ('v1.38.0', ReleaseInfo()),
+            ('v1.39.0', ReleaseInfo()),
+            ('v1.41.1', ReleaseInfo()),
+            ('v1.42.0', ReleaseInfo()),
         ]),
     'php':
         OrderedDict([
@@ -426,6 +506,10 @@ LANG_RELEASE_MATRIX = {
             ('v1.35.0', ReleaseInfo()),
             ('v1.36.3', ReleaseInfo()),
             ('v1.37.0', ReleaseInfo()),
+            ('v1.38.0', ReleaseInfo()),
+            ('v1.39.0', ReleaseInfo()),
+            ('v1.41.1', ReleaseInfo()),
+            ('v1.42.0', ReleaseInfo()),
         ]),
     'csharp':
         OrderedDict([
@@ -470,5 +554,9 @@ LANG_RELEASE_MATRIX = {
             ('v1.35.0', ReleaseInfo()),
             ('v1.36.3', ReleaseInfo()),
             ('v1.37.0', ReleaseInfo()),
+            ('v1.38.1', ReleaseInfo()),
+            ('v1.39.1', ReleaseInfo()),
+            ('v1.41.1', ReleaseInfo()),
+            ('v1.42.0', ReleaseInfo()),
         ]),
 }
