@@ -24,7 +24,7 @@ SMOKETEST = 'smoketest'
 SCALABLE = 'scalable'
 INPROC = 'inproc'
 SWEEP = 'sweep'
-DEFAULT_CATEGORIES = [SCALABLE, SMOKETEST]
+DEFAULT_CATEGORIES = (SCALABLE, SMOKETEST)
 
 SECURE_SECARGS = {
     'use_test_ca': True,
@@ -127,13 +127,13 @@ def _ping_pong_scenario(name,
                         server_threads_per_cq=0,
                         client_threads_per_cq=0,
                         warmup_seconds=WARMUP_SECONDS,
-                        categories=DEFAULT_CATEGORIES,
+                        categories=None,
                         channels=None,
                         outstanding=None,
                         num_clients=None,
                         resource_quota_size=None,
                         messages_per_stream=None,
-                        excluded_poll_engines=[],
+                        excluded_poll_engines=None,
                         minimal_stack=False,
                         offered_load=None):
     """Creates a basic ping pong scenario."""
@@ -162,7 +162,9 @@ def _ping_pong_scenario(name,
             'channel_args': [],
         },
         'warmup_seconds': warmup_seconds,
-        'benchmark_seconds': BENCHMARK_SECONDS
+        'benchmark_seconds': BENCHMARK_SECONDS,
+        'CATEGORIES': list(DEFAULT_CATEGORIES),
+        'EXCLUDED_POLL_ENGINES': [],
     }
     if resource_quota_size:
         scenario['server_config']['resource_quota_size'] = resource_quota_size
@@ -232,10 +234,18 @@ def _ping_pong_scenario(name,
     return scenario
 
 
-class CXXLanguage:
+class Language(object):
 
-    def __init__(self):
-        self.safename = 'cxx'
+    @property
+    def safename(self):
+        return str(self)
+
+
+class CXXLanguage(Language):
+
+    @property
+    def safename(self):
+        return 'cxx'
 
     def worker_cmdline(self):
         return ['cmake/build/qps_worker']
@@ -257,7 +267,7 @@ class CXXLanguage:
             channels=1,
             num_clients=1,
             secure=False,
-            categories=[INPROC] + [SCALABLE])
+            categories=[SWEEP])
 
         yield _ping_pong_scenario(
             'cpp_protobuf_async_streaming_from_client_1channel_1MB',
@@ -271,8 +281,10 @@ class CXXLanguage:
             channels=1,
             num_clients=1,
             secure=False,
-            categories=[SMOKETEST] + [INPROC] + [SCALABLE])
+            categories=[SWEEP])
 
+        # Scenario was added in https://github.com/grpc/grpc/pull/12987, but its purpose is unclear
+        # (beyond excercising some params that other scenarios don't)
         yield _ping_pong_scenario(
             'cpp_protobuf_async_unary_75Kqps_600channel_60Krpcs_300Breq_50Bresp',
             rpc_type='UNARY',
@@ -313,11 +325,13 @@ class CXXLanguage:
                 unconstrained_client='async',
                 use_generic_payload=True,
                 secure=secure,
+                client_threads_per_cq=2,
+                server_threads_per_cq=2,
                 minimal_stack=not secure,
                 categories=smoketest_categories + inproc_categories +
                 [SCALABLE])
 
-            for mps in geometric_progression(1, 20, 10):
+            for mps in geometric_progression(10, 20, 10):
                 yield _ping_pong_scenario(
                     'cpp_generic_async_streaming_qps_unconstrained_%smps_%s' %
                     (mps, secstr),
@@ -385,21 +399,7 @@ class CXXLanguage:
                 secure=secure,
                 client_threads_per_cq=1000000,
                 server_threads_per_cq=1000000,
-                categories=smoketest_categories + inproc_categories +
-                [SCALABLE])
-
-            yield _ping_pong_scenario(
-                'cpp_generic_async_streaming_qps_unconstrained_2waysharedcq_%s'
-                % secstr,
-                rpc_type='STREAMING',
-                client_type='ASYNC_CLIENT',
-                server_type='ASYNC_GENERIC_SERVER',
-                unconstrained_client='async',
-                use_generic_payload=True,
-                secure=secure,
-                client_threads_per_cq=2,
-                server_threads_per_cq=2,
-                categories=inproc_categories + [SCALABLE])
+                categories=[SWEEP])
 
             yield _ping_pong_scenario(
                 'cpp_protobuf_async_streaming_qps_unconstrained_1cq_%s' %
@@ -413,18 +413,6 @@ class CXXLanguage:
                 server_threads_per_cq=1000000,
                 categories=inproc_categories + [SCALABLE])
 
-            yield _ping_pong_scenario(
-                'cpp_protobuf_async_streaming_qps_unconstrained_2waysharedcq_%s'
-                % secstr,
-                rpc_type='STREAMING',
-                client_type='ASYNC_CLIENT',
-                server_type='ASYNC_SERVER',
-                unconstrained_client='async',
-                secure=secure,
-                client_threads_per_cq=2,
-                server_threads_per_cq=2,
-                categories=inproc_categories + [SCALABLE])
-
             yield _ping_pong_scenario(
                 'cpp_protobuf_async_unary_qps_unconstrained_1cq_%s' % secstr,
                 rpc_type='UNARY',
@@ -434,19 +422,6 @@ class CXXLanguage:
                 secure=secure,
                 client_threads_per_cq=1000000,
                 server_threads_per_cq=1000000,
-                categories=smoketest_categories + inproc_categories +
-                [SCALABLE])
-
-            yield _ping_pong_scenario(
-                'cpp_protobuf_async_unary_qps_unconstrained_2waysharedcq_%s' %
-                secstr,
-                rpc_type='UNARY',
-                client_type='ASYNC_CLIENT',
-                server_type='ASYNC_SERVER',
-                unconstrained_client='async',
-                secure=secure,
-                client_threads_per_cq=2,
-                server_threads_per_cq=2,
                 categories=inproc_categories + [SCALABLE])
 
             yield _ping_pong_scenario(
@@ -458,7 +433,8 @@ class CXXLanguage:
                 use_generic_payload=True,
                 async_server_threads=1,
                 minimal_stack=not secure,
-                secure=secure)
+                secure=secure,
+                categories=[SWEEP])
 
             yield _ping_pong_scenario(
                 'cpp_protobuf_async_client_sync_server_unary_qps_unconstrained_%s'
@@ -495,8 +471,7 @@ class CXXLanguage:
                 unconstrained_client='async',
                 secure=secure,
                 minimal_stack=not secure,
-                categories=smoketest_categories + inproc_categories +
-                [SCALABLE])
+                categories=[SWEEP])
 
             yield _ping_pong_scenario(
                 'cpp_protobuf_async_unary_ping_pong_%s_1MB' % secstr,
@@ -540,6 +515,13 @@ class CXXLanguage:
                             minimal_stack=not secure,
                             categories=[SWEEP])
 
+                    maybe_scalable = [SCALABLE]
+                    if rpc_type == 'streaming_from_server' and synchronicity == 'async' and secure:
+                        # protobuf_async_streaming_from_server_qps_unconstrained_secure is very flaky
+                        # and has extremely high variance so running it isn't really useful.
+                        # see b/198275705
+                        maybe_scalable = [SWEEP]
+
                     yield _ping_pong_scenario(
                         'cpp_protobuf_%s_%s_qps_unconstrained_%s' %
                         (synchronicity, rpc_type, secstr),
@@ -549,9 +531,9 @@ class CXXLanguage:
                         unconstrained_client=synchronicity,
                         secure=secure,
                         minimal_stack=not secure,
-                        server_threads_per_cq=3,
-                        client_threads_per_cq=3,
-                        categories=inproc_categories + [SCALABLE])
+                        server_threads_per_cq=2,
+                        client_threads_per_cq=2,
+                        categories=inproc_categories + maybe_scalable)
 
                     # TODO(vjpai): Re-enable this test. It has a lot of timeouts
                     # and hasn't yet been conclusively identified as a test failure
@@ -567,7 +549,7 @@ class CXXLanguage:
                     #     resource_quota_size=500*1024)
 
                     if rpc_type == 'streaming':
-                        for mps in geometric_progression(1, 20, 10):
+                        for mps in geometric_progression(10, 20, 10):
                             yield _ping_pong_scenario(
                                 'cpp_protobuf_%s_%s_qps_unconstrained_%smps_%s'
                                 % (synchronicity, rpc_type, mps, secstr),
@@ -619,10 +601,7 @@ class CXXLanguage:
         return 'c++'
 
 
-class CSharpLanguage:
-
-    def __init__(self):
-        self.safename = str(self)
+class CSharpLanguage(Language):
 
     def worker_cmdline(self):
         return ['tools/run_tests/performance/run_worker_csharp.sh']
@@ -747,10 +726,7 @@ class CSharpLanguage:
         return 'csharp'
 
 
-class PythonLanguage:
-
-    def __init__(self):
-        self.safename = 'python'
+class PythonLanguage(Language):
 
     def worker_cmdline(self):
         return ['tools/run_tests/performance/run_worker_python.sh']
@@ -824,10 +800,7 @@ class PythonLanguage:
         return 'python'
 
 
-class PythonAsyncIOLanguage:
-
-    def __init__(self):
-        self.safename = 'python_asyncio'
+class PythonAsyncIOLanguage(Language):
 
     def worker_cmdline(self):
         return ['tools/run_tests/performance/run_worker_python_asyncio.sh']
@@ -972,11 +945,7 @@ class PythonAsyncIOLanguage:
         return 'python_asyncio'
 
 
-class RubyLanguage:
-
-    def __init__(self):
-        pass
-        self.safename = str(self)
+class RubyLanguage(Language):
 
     def worker_cmdline(self):
         return ['tools/run_tests/performance/run_worker_ruby.sh']
@@ -1037,12 +1006,11 @@ class RubyLanguage:
         return 'ruby'
 
 
-class Php7Language:
+class Php7Language(Language):
 
     def __init__(self, php7_protobuf_c=False):
-        pass
+        super().__init__()
         self.php7_protobuf_c = php7_protobuf_c
-        self.safename = str(self)
 
     def worker_cmdline(self):
         if self.php7_protobuf_c:
@@ -1108,11 +1076,7 @@ class Php7Language:
         return 'php7'
 
 
-class JavaLanguage:
-
-    def __init__(self):
-        pass
-        self.safename = str(self)
+class JavaLanguage(Language):
 
     def worker_cmdline(self):
         return ['tools/run_tests/performance/run_worker_java.sh']
@@ -1212,11 +1176,7 @@ class JavaLanguage:
         return 'java'
 
 
-class GoLanguage:
-
-    def __init__(self):
-        pass
-        self.safename = str(self)
+class GoLanguage(Language):
 
     def worker_cmdline(self):
         return ['tools/run_tests/performance/run_worker_go.sh']
@@ -1297,12 +1257,11 @@ class GoLanguage:
         return 'go'
 
 
-class NodeLanguage:
+class NodeLanguage(Language):
 
     def __init__(self, node_purejs=False):
-        pass
+        super().__init__()
         self.node_purejs = node_purejs
-        self.safename = str(self)
 
     def worker_cmdline(self):
         fixture = 'native_js' if self.node_purejs else 'native_native'
