@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # Copyright 2015 gRPC authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -34,16 +34,18 @@ import socket
 import subprocess
 import sys
 import tempfile
-import traceback
 import time
-from six.moves import urllib
+import traceback
 import uuid
+
 import six
+from six.moves import urllib
 
 import python_utils.jobset as jobset
 import python_utils.report_utils as report_utils
-import python_utils.watch_dirs as watch_dirs
 import python_utils.start_port_server as start_port_server
+import python_utils.watch_dirs as watch_dirs
+
 try:
     from python_utils.upload_test_results import upload_results_to_bq
 except (ImportError):
@@ -176,22 +178,18 @@ _PythonConfigVars = collections.namedtuple('_ConfigVars', [
     'venv_relative_python',
     'toolchain',
     'runner',
-    'test_name',
-    'iomgr_platform',
 ])
 
 
 def _python_config_generator(name, major, minor, bits, config_vars):
-    name += '_' + config_vars.iomgr_platform
-    return PythonConfig(
-        name, config_vars.shell + config_vars.builder +
-        config_vars.builder_prefix_arguments +
-        [_python_pattern_function(major=major, minor=minor, bits=bits)] +
-        [name] + config_vars.venv_relative_python + config_vars.toolchain,
-        config_vars.shell + config_vars.runner + [
-            os.path.join(name, config_vars.venv_relative_python[0]),
-            config_vars.test_name
-        ])
+    build = (config_vars.shell + config_vars.builder +
+             config_vars.builder_prefix_arguments +
+             [_python_pattern_function(major=major, minor=minor, bits=bits)] +
+             [name] + config_vars.venv_relative_python + config_vars.toolchain)
+    run = (config_vars.shell + config_vars.runner + [
+        os.path.join(name, config_vars.venv_relative_python[0]),
+    ])
+    return PythonConfig(name, build, run)
 
 
 def _pypy_config_generator(name, major, config_vars):
@@ -271,23 +269,6 @@ class CLanguage(object):
                 # see https://github.com/grpc/grpc/blob/b5b8578b3f8b4a9ce61ed6677e19d546e43c5c68/tools/run_tests/artifacts/artifact_targets.py#L253
                 self._cmake_configure_extra_args.append('-DOPENSSL_NO_ASM=ON')
 
-        if args.iomgr_platform == "uv":
-            cflags = '-DGRPC_UV -DGRPC_CUSTOM_IOMGR_THREAD_CHECK -DGRPC_CUSTOM_SOCKET '
-            try:
-                cflags += subprocess.check_output(
-                    ['pkg-config', '--cflags', 'libuv']).strip() + ' '
-            except (subprocess.CalledProcessError, OSError):
-                pass
-            try:
-                ldflags = subprocess.check_output(
-                    ['pkg-config', '--libs', 'libuv']).strip() + ' '
-            except (subprocess.CalledProcessError, OSError):
-                ldflags = '-luv '
-            self._make_options += [
-                'EXTRA_CPPFLAGS={}'.format(cflags),
-                'EXTRA_LDLIBS={}'.format(ldflags)
-            ]
-
     def test_specs(self):
         out = []
         binaries = get_c_tests(self.args.travis, self.test_lang)
@@ -299,8 +280,6 @@ class CLanguage(object):
             polling_strategies = (_POLLING_STRATEGIES.get(
                 self.platform, ['all']) if target.get('uses_polling', True) else
                                   ['none'])
-            if self.args.iomgr_platform == 'uv':
-                polling_strategies = ['all']
             for polling_strategy in polling_strategies:
                 env = {
                     'GRPC_DEFAULT_SSL_ROOTS_FILE_PATH':
@@ -441,8 +420,10 @@ class CLanguage(object):
 
     def pre_build_steps(self):
         if self.platform == 'windows':
-            return [['tools\\run_tests\\helper_scripts\\pre_build_cmake.bat'] +
-                    self._cmake_configure_extra_args]
+            return [[
+                'tools\\run_tests\\helper_scripts\\pre_build_cmake.bat',
+                '-DgRPC_BUILD_MSVC_MP_COUNT=%d' % args.jobs
+            ] + self._cmake_configure_extra_args]
         elif self._use_cmake:
             return [['tools/run_tests/helper_scripts/pre_build_cmake.sh'] +
                     self._cmake_configure_extra_args]
@@ -477,24 +458,24 @@ class CLanguage(object):
             # so we only allow the non-specific choices.
             _check_compiler(compiler, ['default', 'cmake'])
 
-        if compiler == 'gcc4.9' or compiler == 'default' or compiler == 'cmake':
-            return ('jessie', [])
-        elif compiler == 'gcc5.3':
-            return ('ubuntu1604', [])
-        elif compiler == 'gcc7.4':
-            return ('ubuntu1804', [])
+        if compiler == 'default' or compiler == 'cmake':
+            return ('debian9', [])
+        elif compiler == 'gcc4.9':
+            return ('gcc_4.9', [])
         elif compiler == 'gcc8.3':
-            return ('buster', [])
+            return ('debian10', [])
+        elif compiler == 'gcc8.3_openssl102':
+            return ('debian10_openssl102', [
+                "-DgRPC_SSL_PROVIDER=package",
+            ])
+        elif compiler == 'gcc11':
+            return ('gcc_11', [])
         elif compiler == 'gcc_musl':
             return ('alpine', [])
-        elif compiler == 'clang4.0':
-            return ('ubuntu1604',
-                    self._clang_cmake_configure_extra_args(
-                        version_suffix='-4.0'))
-        elif compiler == 'clang5.0':
-            return ('ubuntu1604',
-                    self._clang_cmake_configure_extra_args(
-                        version_suffix='-5.0'))
+        elif compiler == 'clang4':
+            return ('clang_4', self._clang_cmake_configure_extra_args())
+        elif compiler == 'clang12':
+            return ('clang_12', self._clang_cmake_configure_extra_args())
         else:
             raise Exception('Compiler %s not supported.' % compiler)
 
@@ -607,7 +588,7 @@ class Php7Language(object):
         return 'Makefile'
 
     def dockerfile_dir(self):
-        return 'tools/dockerfile/test/php7_jessie_%s' % _docker_arch_suffix(
+        return 'tools/dockerfile/test/php7_stretch_%s' % _docker_arch_suffix(
             self.args.arch)
 
     def __str__(self):
@@ -622,13 +603,17 @@ class PythonConfig(
 class PythonLanguage(object):
 
     _TEST_SPECS_FILE = {
-        'native': 'src/python/grpcio_tests/tests/tests.json',
-        'gevent': 'src/python/grpcio_tests/tests/tests.json',
-        'asyncio': 'src/python/grpcio_tests/tests_aio/tests.json',
+        'native': ['src/python/grpcio_tests/tests/tests.json'],
+        'gevent': [
+            'src/python/grpcio_tests/tests/tests.json',
+            'src/python/grpcio_tests/tests_gevent/tests.json',
+        ],
+        'asyncio': ['src/python/grpcio_tests/tests_aio/tests.json'],
     }
-    _TEST_FOLDER = {
-        'native': 'test',
-        'gevent': 'test',
+
+    _TEST_COMMAND = {
+        'native': 'test_lite',
+        'gevent': 'test_gevent',
         'asyncio': 'test_aio',
     }
 
@@ -639,26 +624,36 @@ class PythonLanguage(object):
 
     def test_specs(self):
         # load list of known test suites
-        with open(self._TEST_SPECS_FILE[
-                self.args.iomgr_platform]) as tests_json_file:
-            tests_json = json.load(tests_json_file)
-        environment = dict(_FORCE_ENVIRON_FOR_WRAPPERS)
-        # TODO(https://github.com/grpc/grpc/issues/21401) Fork handlers is not
-        # designed for non-native IO manager. It has a side-effect that
-        # overrides threading settings in C-Core.
-        if args.iomgr_platform != 'native':
-            environment['GRPC_ENABLE_FORK_SUPPORT'] = '0'
-        return [
-            self.config.job_spec(
-                config.run,
-                timeout_seconds=8 * 60,
-                environ=dict(GRPC_PYTHON_TESTRUNNER_FILTER=str(suite_name),
-                             **environment),
-                shortname='%s.%s.%s' %
-                (config.name, self._TEST_FOLDER[self.args.iomgr_platform],
-                 suite_name),
-            ) for suite_name in tests_json for config in self.pythons
-        ]
+        jobs = []
+        for io_platform in self._TEST_SPECS_FILE:
+            test_cases = []
+            for tests_json_file_name in self._TEST_SPECS_FILE[io_platform]:
+                with open(tests_json_file_name) as tests_json_file:
+                    test_cases.extend(json.load(tests_json_file))
+
+            environment = dict(_FORCE_ENVIRON_FOR_WRAPPERS)
+            # TODO(https://github.com/grpc/grpc/issues/21401) Fork handlers is not
+            # designed for non-native IO manager. It has a side-effect that
+            # overrides threading settings in C-Core.
+            if io_platform != 'native':
+                environment['GRPC_ENABLE_FORK_SUPPORT'] = '0'
+            for python_config in self.pythons:
+                # TODO(https://github.com/grpc/grpc/issues/23784) allow gevent
+                # to run on later version once issue solved.
+                if io_platform == 'gevent' and python_config.name != 'py36':
+                    continue
+                jobs.extend([
+                    self.config.job_spec(
+                        python_config.run + [self._TEST_COMMAND[io_platform]],
+                        timeout_seconds=8 * 60,
+                        environ=dict(
+                            GRPC_PYTHON_TESTRUNNER_FILTER=str(test_case),
+                            **environment),
+                        shortname='%s.%s.%s' %
+                        (python_config.name, io_platform, test_case),
+                    ) for test_case in test_cases
+                ])
+        return jobs
 
     def pre_build_steps(self):
         return []
@@ -687,9 +682,7 @@ class PythonLanguage(object):
 
     def _python_manager_name(self):
         """Choose the docker image to use based on python version."""
-        if self.args.compiler in [
-                'python2.7', 'python3.5', 'python3.6', 'python3.7', 'python3.8'
-        ]:
+        if self.args.compiler in ['python3.6', 'python3.7', 'python3.8']:
             return 'stretch_' + self.args.compiler[len('python'):]
         elif self.args.compiler == 'python_alpine':
             return 'alpine'
@@ -698,6 +691,11 @@ class PythonLanguage(object):
 
     def _get_pythons(self, args):
         """Get python runtimes to test with, based on current platform, architecture, compiler etc."""
+        if args.iomgr_platform != 'native':
+            raise ValueError(
+                'Python builds no longer differentiate IO Manager platforms, please use "native"'
+            )
+
         if args.arch == 'x86':
             bits = '32'
         else:
@@ -722,35 +720,13 @@ class PythonLanguage(object):
             venv_relative_python = ['bin/python']
             toolchain = ['unix']
 
-        # Selects the corresponding testing mode.
-        # See src/python/grpcio_tests/commands.py for implementation details.
-        if args.iomgr_platform == 'native':
-            test_command = 'test_lite'
-        elif args.iomgr_platform == 'gevent':
-            test_command = 'test_gevent'
-        elif args.iomgr_platform == 'asyncio':
-            test_command = 'test_aio'
-        else:
-            raise ValueError('Unsupported IO Manager platform: %s' %
-                             args.iomgr_platform)
         runner = [
             os.path.abspath('tools/run_tests/helper_scripts/run_python.sh')
         ]
 
         config_vars = _PythonConfigVars(shell, builder,
                                         builder_prefix_arguments,
-                                        venv_relative_python, toolchain, runner,
-                                        test_command, args.iomgr_platform)
-        python27_config = _python_config_generator(name='py27',
-                                                   major='2',
-                                                   minor='7',
-                                                   bits=bits,
-                                                   config_vars=config_vars)
-        python35_config = _python_config_generator(name='py35',
-                                                   major='3',
-                                                   minor='5',
-                                                   bits=bits,
-                                                   config_vars=config_vars)
+                                        venv_relative_python, toolchain, runner)
         python36_config = _python_config_generator(name='py36',
                                                    major='3',
                                                    minor='6',
@@ -766,6 +742,16 @@ class PythonLanguage(object):
                                                    minor='8',
                                                    bits=bits,
                                                    config_vars=config_vars)
+        python39_config = _python_config_generator(name='py39',
+                                                   major='3',
+                                                   minor='9',
+                                                   bits=bits,
+                                                   config_vars=config_vars)
+        python310_config = _python_config_generator(name='py310',
+                                                    major='3',
+                                                    minor='10',
+                                                    bits=bits,
+                                                    config_vars=config_vars)
         pypy27_config = _pypy_config_generator(name='pypy',
                                                major='2',
                                                config_vars=config_vars)
@@ -773,62 +759,42 @@ class PythonLanguage(object):
                                                major='3',
                                                config_vars=config_vars)
 
-        if args.iomgr_platform == 'asyncio':
-            if args.compiler not in ('default', 'python3.6', 'python3.7',
-                                     'python3.8'):
-                raise Exception(
-                    'Compiler %s not supported with IO Manager platform: %s' %
-                    (args.compiler, args.iomgr_platform))
-
         if args.compiler == 'default':
             if os.name == 'nt':
-                if args.iomgr_platform == 'gevent':
-                    # TODO(https://github.com/grpc/grpc/issues/23784) allow
-                    # gevent to run on later version once issue solved.
-                    return (python36_config,)
-                else:
-                    return (python38_config,)
+                return (python38_config,)
+            elif os.uname()[0] == 'Darwin':
+                # NOTE(rbellevi): Testing takes significantly longer on
+                # MacOS, so we restrict the number of interpreter versions
+                # tested.
+                return (python38_config,)
             else:
-                if args.iomgr_platform == 'asyncio':
-                    return (python36_config, python38_config)
-                elif os.uname()[0] == 'Darwin':
-                    # NOTE(rbellevi): Testing takes significantly longer on
-                    # MacOS, so we restrict the number of interpreter versions
-                    # tested.
-                    return (
-                        python27_config,
-                        python38_config,
-                    )
-                else:
-                    return (
-                        python27_config,
-                        python35_config,
-                        python37_config,
-                        python38_config,
-                    )
-        elif args.compiler == 'python2.7':
-            return (python27_config,)
-        elif args.compiler == 'python3.5':
-            return (python35_config,)
+                return (
+                    python36_config,
+                    python38_config,
+                )
         elif args.compiler == 'python3.6':
             return (python36_config,)
         elif args.compiler == 'python3.7':
             return (python37_config,)
         elif args.compiler == 'python3.8':
             return (python38_config,)
+        elif args.compiler == 'python3.9':
+            return (python39_config,)
+        elif args.compiler == 'python3.10':
+            return (python310_config,)
         elif args.compiler == 'pypy':
             return (pypy27_config,)
         elif args.compiler == 'pypy3':
             return (pypy32_config,)
         elif args.compiler == 'python_alpine':
-            return (python27_config,)
+            return (python38_config,)
         elif args.compiler == 'all_the_cpythons':
             return (
-                python27_config,
-                python35_config,
                 python36_config,
                 python37_config,
                 python38_config,
+                python39_config,
+                python310_config,
             )
         else:
             raise Exception('Compiler %s not supported.' % args.compiler)
@@ -917,7 +883,7 @@ class CSharpLanguage(object):
             self._cmake_arch_option = 'x64'
         else:
             _check_compiler(self.args.compiler, ['default', 'coreclr'])
-            self._docker_distro = 'stretch'
+            self._docker_distro = 'buster'
 
     def test_specs(self):
         with open('src/csharp/tests.json') as f:
@@ -929,7 +895,7 @@ class CSharpLanguage(object):
         assembly_extension = '.exe'
 
         if self.args.compiler == 'coreclr':
-            assembly_subdir += '/netcoreapp2.1'
+            assembly_subdir += '/netcoreapp3.1'
             runtime_cmd = ['dotnet', 'exec']
             assembly_extension = '.dll'
         else:
@@ -1107,6 +1073,13 @@ class ObjCLanguage(object):
                 shortname='ios-test-cfstream-tests',
                 cpu_cost=1e6,
                 environ=_FORCE_ENVIRON_FOR_WRAPPERS))
+        out.append(
+            self.config.job_spec(
+                ['src/objective-c/tests/CoreTests/build_and_run_tests.sh'],
+                timeout_seconds=60 * 60,
+                shortname='ios-test-core-tests',
+                cpu_cost=1e6,
+                environ=_FORCE_ENVIRON_FOR_WRAPPERS))
         # TODO: replace with run_one_test_bazel.sh when Bazel-Xcode is stable
         out.append(
             self.config.job_spec(['src/objective-c/tests/run_one_test.sh'],
@@ -1375,11 +1348,14 @@ argp.add_argument('-f',
                   default=False,
                   action='store_const',
                   const=True)
-argp.add_argument('-t',
-                  '--travis',
-                  default=False,
-                  action='store_const',
-                  const=True)
+argp.add_argument(
+    '-t',
+    '--travis',
+    default=False,
+    action='store_const',
+    const=True,
+    help='When set, indicates that the script is running on CI (= not locally).'
+)
 argp.add_argument('--newline_on_success',
                   default=False,
                   action='store_const',
@@ -1421,17 +1397,18 @@ argp.add_argument(
     choices=[
         'default',
         'gcc4.9',
-        'gcc5.3',
-        'gcc7.4',
         'gcc8.3',
+        'gcc8.3_openssl102',
+        'gcc11',
         'gcc_musl',
-        'clang4.0',
-        'clang5.0',
+        'clang4',
+        'clang12',
         'python2.7',
         'python3.5',
         'python3.6',
         'python3.7',
         'python3.8',
+        'python3.9',
         'pypy',
         'pypy3',
         'python_alpine',
@@ -1449,7 +1426,7 @@ argp.add_argument(
     'Selects compiler to use. Allowed values depend on the platform and language.'
 )
 argp.add_argument('--iomgr_platform',
-                  choices=['native', 'uv', 'gevent', 'asyncio'],
+                  choices=['native', 'gevent', 'asyncio'],
                   default='native',
                   help='Selects iomgr platform to build on')
 argp.add_argument('--build_only',
@@ -1608,7 +1585,7 @@ if args.use_docker:
         dockerfile_dir = next(iter(dockerfile_dirs))
 
     child_argv = [arg for arg in sys.argv if not arg == '--use_docker']
-    run_tests_cmd = 'python tools/run_tests/run_tests.py %s' % ' '.join(
+    run_tests_cmd = 'python3 tools/run_tests/run_tests.py %s' % ' '.join(
         child_argv[1:])
 
     env = os.environ.copy()
@@ -1617,8 +1594,6 @@ if args.use_docker:
     env['DOCKER_RUN_SCRIPT'] = 'tools/run_tests/dockerize/docker_run_tests.sh'
     if args.xml_report:
         env['XML_REPORT'] = args.xml_report
-    if not args.travis:
-        env['TTY_FLAG'] = '-t'  # enables Ctrl-C when not on Jenkins.
 
     subprocess.check_call(
         'tools/run_tests/dockerize/build_docker_and_run_tests.sh',
